!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!
! forcing.f90	Subroutine to calculate forcing terms in PV equation.
!		Subroutine calculates:
!					tend = forcing (psi_in, q_in).
!		Forcing consists of following options:
!		(i)   Forcing of *perturbation* PV by the *equilibrium* flow.
!		(ii)  Relaxation of PPV to specified field.
!		(iii) Stochastic forcing to represent sub grid-scale effects.
!		(iv)  Forcing by bottom and top topography.
!
!		Called by PROGRAM qgam.
! 
! twnh 7Jan01 & pdw 2Jul02
!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

SUBROUTINE forcing(q_in, psi_in, tend)

! load global variables
USE dyn_vars
USE grid_params
USE phys_params

IMPLICIT NONE

! declaration of local variables
REAL (KIND=wp), DIMENSION(N_rad, N_azim, N_layer), INTENT(IN)		:: q_in, psi_in			! input fields
REAL (KIND=wp), DIMENSION(N_rad, N_azim, N_layer), INTENT(INOUT)	:: tend				! tendency in q field
REAL (KIND=wp), DIMENSION(N_rad, N_azim, N_layer)			:: dq_dtheta, dpsi_dtheta	! temporary variables

WRITE(*,*) ' forcing...'

! (i) forcing of perturbation PV by the equilibrium flow (currently for 2 layers only)
IF(N_layer > 2) THEN
	WRITE(*,*) '   ERROR!  N_layer > 2 is not tested in forcing.f90'
	STOP
ELSE 
	CALL dtheta(psi_in, dpsi_dtheta)					! get azimuthal gradients of psi
	tend(:,:,1) = tend(:,:,1) + forcing_coeff*dpsi_dtheta(:,:,1)
	tend(:,:,2) = tend(:,:,2) - forcing_coeff*dpsi_dtheta(:,:,2)
	CALL dtheta(q_in, dq_dtheta)						! get azimuthal gradients of q 
	tend(:,:,1) = tend(:,:,1) - delta_omega(1)*dq_dtheta(:,:,1)
	tend(:,:,2) = tend(:,:,2) - delta_omega(2)*dq_dtheta(:,:,2)
ENDIF

! (ii) relaxation of PPV to specified field
IF (relax_type == 2 .OR. relax_type == 3) THEN
	tend = tend + relax_rate*(q_relax - q_in)
ENDIF

! (iii) stochastic forcing to represent sub grid-scale effects
noise_amp = noise_amp + d_dt_noise_amp * delta_t				! to quasi-continuously increment amplitude
CALL RANDOM_NUMBER(random)
random = noise_amp * (2.0_wp * random - 1.0_wp)					! change interval
random = random - SUM(area_element*random)/area					! ensure mean zero
tend(:,:,1) = tend(:,:,1) + random(:,:)						! } 
tend(:,:,2) = tend(:,:,2) - random(:,:)						! } purely baroclinic

! (iv) forcing by bottom and top topography
tend = tend + top_forc_coeff * dpsi_dtheta

END SUBROUTINE forcing
