!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!
! init_solver.f90	Initialize the solver by allocating and evaluating those global
!			arrays defined in the solver_vars module.
!
!			Called by PROGRAM qgam.
! 
! pdw & twnh 13Aug01
!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

SUBROUTINE init_solver

! NAG routines used
USE nag_fft, ONLY : nag_fft_trig
USE nag_gen_bnd_lin_sys, ONLY : nag_gen_bnd_lin_fac

! load global variables
USE grid_params
USE phys_params
USE solver_vars

IMPLICIT NONE

! declaration of local variables
COMPLEX (KIND=wp), DIMENSION(N_azim/2+1, 3, N_rad)	:: a_bt, a_bc			! N_azim/2+1 tridiagonal matrices
REAL (KIND=wp), DIMENSION(N_azim/2+1)			:: rcond_bt, rcond_bc		! N_azim/2+1 reciprocal condition numbers
INTEGER							:: i, k				! indices

WRITE(*,*) ' init_solver: Initializing Helmholtz solver:'

! allocate storage space for permanent arrays
ALLOCATE ( 	fft_coeffs(2*N_azim), 			&
		a_bt_fac(N_azim/2+1, 4, N_rad), 	&
		a_bc_fac(N_azim/2+1, 4, N_rad), 	&
		pivot_bt(N_azim/2+1, N_rad), 		&
		pivot_bc(N_azim/2+1, N_rad)		)

! evaluate vector fft_coeffs of trigonometric coefficients for later use by the NAG FFT routines
WRITE (*,*) '              Evaluating trigonometric coefficients...'
CALL nag_fft_trig(fft_coeffs)

! evaluate the tridiagonal matrices a_bt and a_bc, using the compact storage scheme required by the NAG band matrix routines
WRITE (*,*) '              Evaluating tridiagonal matrices...'
a_bt = 0.0_wp
a_bc = 0.0_wp
DO k=1, N_azim/2+1										! Fourier component is k-1

	DO i=3, N_rad										! fills in first row of each matrix
		a_bt(k,1,i) = 1.0_wp + delta_r/(2.0_wp*rad_dist(i-1))
		a_bc(k,1,i) = 1.0_wp + delta_r/(2.0_wp*rad_dist(i-1))
	END DO

	DO i=2, N_rad-1										! fills in second row of each matrix
		a_bt(k,2,i) = -2.0_wp - (delta_r**2)*(lambda_bt + (REAL(k-1,KIND=wp)/rad_dist(i))**2)
		a_bc(k,2,i) = -2.0_wp - (delta_r**2)*(lambda_bc + (REAL(k-1,KIND=wp)/rad_dist(i))**2)
	END DO

	DO i=1, N_rad-2										! fills in third row of each matrix
		a_bt(k,3,i) = 1.0_wp - delta_r/(2.0_wp*rad_dist(i+1))
		a_bc(k,3,i) = 1.0_wp - delta_r/(2.0_wp*rad_dist(i+1))
	END DO

	IF (k == 1) THEN			! boundary conditions for mean flow correction (i.e. zero Fourier component)
		a_bt(k,2,1) = -1.0_wp
		a_bt(k,1,2) = 1.0_wp
		a_bt(k,3,N_rad-1) = 0.0_wp
		a_bt(k,2,N_rad) = 1.0_wp  
		a_bc(k,2,1) = -1.0_wp
		a_bc(k,1,2) = 1.0_wp
		a_bc(k,3,N_rad-1) = 1.0_wp
		a_bc(k,2,N_rad) = -1.0_wp
	ELSE					! boundary conditions for waves (i.e. non-zero Fourier components)
		a_bt(k,2,1) = 1.0_wp
		a_bt(k,1,2) = 0.0_wp
		a_bt(k,3,N_rad-1) = 0.0_wp
		a_bt(k,2,N_rad) = 1.0_wp
		a_bc(k,2,1) = 1.0_wp
		a_bc(k,1,2) = 0.0_wp
		a_bc(k,3,N_rad-1) = 0.0_wp
		a_bc(k,2,N_rad) = 1.0_wp
	ENDIF

END DO

! factorize the tridiagonal matrices to obtain a_bt_fac, a_bc_fac, pivot_bt, pivot_bc for later use by the NAG band matrix solver
WRITE (*,*) '              Factorizing tridiagonal matrices...'
DO k=1, N_azim/2+1
	CALL nag_gen_bnd_lin_fac(1, a_bt(k,:,:), a_bt_fac(k,:,:), pivot_bt(k,:), rcond_inf=rcond_bt(k))
	CALL nag_gen_bnd_lin_fac(1, a_bc(k,:,:), a_bc_fac(k,:,:), pivot_bc(k,:), rcond_inf=rcond_bc(k))
END DO

WRITE (*,*) '              Estimates of tridiagonal matrix condition numbers (in the infinity-norm) :'
WRITE (*,*) '              Fourier component        Barotropic             Baroclinic'
DO k=1, N_azim/2+1
	WRITE (*,*) '                    ', k-1, '          ', 1.0_wp/rcond_bt(k), 1.0_wp/rcond_bc(k)
END DO

END SUBROUTINE init_solver
